/*jslint nomen: true, maxlen: 200, indent: 2, maxerr: 100*/
/*jshint esversion: 6*/
/*global window, document, require, URL, rJS*/

(function (window, document, rJS) {
  "use strict";
  let repo = "nexedi/slapos";
  let branch = "master";
  const baseUrl = `https://lab.nexedi.com/${repo}/raw/${branch}/software/erp5/`;
  const mainSchema = "instance-erp5-input-schema.json";

  let monacoSchemaEditor;
  let monacoEditor;
// monaco should ignore edit event when it's caused by rjs_json_form
  let monacoIgnoreEvent = false;
  /*
   based on https://microsoft.github.io/monaco-editor/playground.html#extending-language-services-configure-json-defaults
   and https://codepen.io/maxvrko/full/XVxVyY
  */

  require.config({
    paths: {vs: "https://unpkg.com/monaco-editor@0.15.5/min/vs"}
  });
  let proxy = URL.createObjectURL(
    new Blob(
      [`
self.MonacoEnvironment = {
  baseUrl: 'https://unpkg.com/monaco-editor@0.15.5/min/'
};
importScripts('https://unpkg.com/monaco-editor@0.15.5/min/vs/base/worker/workerMain.js');
`
      ],
      {type: "text/javascript"}
    )
  );

  window.MonacoEnvironment = {getWorkerUrl: () => proxy};

  rJS(window)
    .allowPublicAcquisition("notifyValid", function (arr, scope) {
    })
    .allowPublicAcquisition("notifyInvalid", function (arr, scope) {
    })
    .allowPublicAcquisition("notifyChange", function (arr, scope) {
      return this.getDeclaredGadget("rjs_json_form").then(function (g) {
        return g.getContent().then(function (content) {
          monacoIgnoreEvent = true;
          monacoEditor
            .getModel()
            .setValue(JSON.stringify(JSON.parse(content.rjs_json_form), null, 4));
          monacoIgnoreEvent = false;
        });
      });
    })
    .ready(function () {
      let gadget = this;

      require(["vs/editor/editor.main"], function (monaco) {
        let modelUri = monaco.Uri.parse("tmp://xxx.json"); // a made up unique URI for our model
        let model = monaco.editor.createModel("{\n}", "json", modelUri);

        const getSchema = (schemaUrl, mappedUrl)=> {
          return fetch(schemaUrl)
            .then(resp => {
              return resp.json();
            })
            .then(s => {
              return {
                uri: mappedUrl || schemaUrl,
                schema: s
              };
            });
        };

        Promise.all([
          getSchema(baseUrl + mainSchema).then(s => {
            s.fileMatch = [modelUri.toString()]; // this is the main schema associated to the model
            return s;
          }),
          getSchema("../../json-schema/schema7.json", "http://json-schema.org/draft-07/schema")
        ]).then(schemas => {
          gadget.getDeclaredGadget("rjs_json_form").then(g => {
            return g.render({
              key: "rjs_json_form",
              schema: schemas[0].schema,
              schema_url: schemas[0].uri,
              value: {}
            });
          });

          monaco.languages.json.jsonDefaults.setDiagnosticsOptions({
            validate: true,
            enableSchemaRequest: true,
            schemas: schemas
          });

          monacoSchemaEditor = monaco.editor.create(
            document.getElementById("monaco-schema"),
            {
              model: monaco.editor.createModel(JSON.stringify(schemas[0].schema, null, 2), "json", schemas[0].uri)
            }
          );

          monacoSchemaEditor.onDidChangeModelContent(e => {
            if (!monacoIgnoreEvent) {
              console.log("schema changed", monacoEditor.getValue());
              let parsed;
              try {
                parsed = JSON.parse(monacoSchemaEditor.getValue());
              } catch (error) {
                console.error("parse error, ignoring changes from monaco", error);
                return;
              }
              gadget.getDeclaredGadget("rjs_json_form").then(g => {
                return g.render({
                  key: "rjs_json_form",
                  schema: parsed,
                  schema_url: schemas[0].uri,
                });
              });
            }
          });

          monacoEditor = monaco.editor.create(
            document.getElementById("monaco-editor"),
            {
              model: model
            }
          );

          monacoEditor.onDidChangeModelContent(e => {
            if (!monacoIgnoreEvent) {
              console.log("monaco changed", monacoEditor.getValue());
              let parsed;
              try {
                parsed = JSON.parse(monacoEditor.getValue());
              } catch (error) {
                console.error("parse error, ignoring changes from monaco", error);
                return;
              }
              gadget.getDeclaredGadget("rjs_json_form").then(g => {
                return g.render({
                  key: "rjs_json_form",
                  schema: schemas[0].schema,
                  schema_url: schemas[0].uri,
                  value: parsed
                });
              });
            }
          });
        });
      });
    });
}(window, document, rJS));